package com.car.frpc_android.ui;

import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.car.frpc_android.BaseActivity;
import com.car.frpc_android.R;
import com.car.frpc_android.databinding.ActivityLogcatBinding;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.HashSet;
import java.util.LinkedHashSet;

import io.reactivex.Observable;
import io.reactivex.ObservableOnSubscribe;
import io.reactivex.Observer;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

public class LogcatActivity extends BaseActivity<ActivityLogcatBinding> {

    @Override
    public void initView() {
        initToolbar();
        readLog(false);
    }

    private void readLog(boolean flush) {
        HashSet<String> lst = new LinkedHashSet<>();
        lst.add("logcat");
        lst.add("-d");
        lst.add("-v");
        lst.add("time");
        lst.add("-s");
        lst.add("GoLog,com.car.frpc_android.util.FrpcService");
        lst.add("hzshkj");
        Observable.create((ObservableOnSubscribe<String>) emitter -> {

                    if (flush) {
                        HashSet<String> lst2 = new LinkedHashSet<>();
                        lst2.add("logcat");
                        lst2.add("-c");
                        Process process = Runtime.getRuntime().exec(lst2.toArray(new String[0]));
                        process.waitFor();
                    }

                    Process process = Runtime.getRuntime().exec(lst.toArray(new String[0]));

                    InputStreamReader in = new InputStreamReader(process.getInputStream());
                    BufferedReader bufferedReader = new BufferedReader(in);

                    String line = null;
                    while ((line = bufferedReader.readLine()) != null) {
                        emitter.onNext(line);
                    }
                    in.close();
                    bufferedReader.close();
                    emitter.onComplete();
                }).subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<String>() {
                    @Override
                    public void onSubscribe(Disposable d) {

                    }

                    @Override
                    public void onNext(String s) {
                        b.tvLogcat.append(s);
                        b.tvLogcat.append("\r\n");
                        b.svLogcat.fullScroll(View.FOCUS_DOWN);
                    }

                    @Override
                    public void onError(Throwable e) {
                        e.printStackTrace();
                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }

    private void initToolbar() {
        setSupportActionBar(b.toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        b.toolbar.setNavigationOnClickListener(v -> finish());
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_logcat, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.copy:
                setClipboard(b.tvLogcat.getText().toString());
                Toast.makeText(this, R.string.copySuccess, Toast.LENGTH_SHORT).show();
                break;
            case R.id.delete:
                readLog(true);
                b.tvLogcat.setText("");
                break;

        }
        return super.onOptionsItemSelected(item);
    }

    public void setClipboard(String content) {
        try {
            ClipboardManager cmb = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
            ClipData clipData = ClipData.newPlainText("logcat", content);
            cmb.setPrimaryClip(clipData);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
