package com.car.frpc_android.util;

import static com.car.frpc_android.Config.BREATH_TAG;
import static com.car.frpc_android.Config.SMS_UPLOAD_TAG;
import static com.car.frpc_android.Config.WEBSOCKET_STATUS_TAG;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;

import androidx.annotation.Nullable;
import androidx.core.app.NotificationCompat;
import androidx.lifecycle.Observer;

import com.car.frpc_android.R;
import com.jeremyliao.liveeventbus.LiveEventBus;

import java.util.HashMap;

public class ForegroundService extends Service {
    private static final int NOTIFICATION_ID = 1;
    private static final String CHANNEL_ID = "channel_id";

    private NotificationManager notificationManager;
    private NotificationCompat.Builder notificationBuilder;

    private HashMap<String, String> map;
    private Observer<String> observer1;
    private Observer<String> observer2;
    private Observer<String> observer3;

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        throw new UnsupportedOperationException("Not yet implemented");
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onCreate() {
        super.onCreate();
        // 获取 NotificationManager
        map = new HashMap<>();
        notificationManager = getSystemService(NotificationManager.class);
        // 创建通知渠道（仅针对 Android O 及以上版本）
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationChannel channel = new NotificationChannel(CHANNEL_ID, "channel_name", NotificationManager.IMPORTANCE_LOW);
            if (notificationManager != null) {
                notificationManager.createNotificationChannel(channel);
            }
        }
        // 创建通知
        notificationBuilder = new NotificationCompat.Builder(this, CHANNEL_ID)
                .setSmallIcon(R.mipmap.ic_launcher)
                .setContentTitle(getString(R.string.current_status))
                .setContentText(getString(R.string.no_message))
                .setPriority(NotificationCompat.PRIORITY_LOW);

        startForeground(NOTIFICATION_ID, notificationBuilder.build());
        observer1 = s -> {
            map.put(BREATH_TAG, s);
            updateNotification(map);
        };
        observer2 = s -> {
            map.put(WEBSOCKET_STATUS_TAG, s);
            updateNotification(map);

        };
        observer3 = s -> {
            map.put(SMS_UPLOAD_TAG, s);
            updateNotification(map);

        };

        LiveEventBus
                .get(BREATH_TAG, String.class)
                .observeForever(observer1);
        LiveEventBus
                .get(WEBSOCKET_STATUS_TAG, String.class)
                .observeForever(observer2);
        LiveEventBus
                .get(SMS_UPLOAD_TAG, String.class)
                .observeForever(observer3);


    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        stopForeground(true);
        LiveEventBus.get(BREATH_TAG, String.class).removeObserver(observer1);
        LiveEventBus.get(WEBSOCKET_STATUS_TAG, String.class).removeObserver(observer2);
        LiveEventBus.get(SMS_UPLOAD_TAG, String.class).removeObserver(observer3);
    }

    private void updateNotification(HashMap map) {
        if (notificationBuilder != null) {
            String contentText = map.get(BREATH_TAG)
                    + ","
                    + map.get(WEBSOCKET_STATUS_TAG)
                    + ","
                    + map.get(SMS_UPLOAD_TAG);
            notificationBuilder.setContentText(contentText);
            notificationManager.notify(NOTIFICATION_ID, notificationBuilder.build());
        }
    }
}
